"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = y[op[0] & 2 ? "return" : op[0] ? "throw" : "next"]) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [0, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var fs = require("fs");
var fsPath = require("path");
var _ = require("lodash");
var VEvent_1 = require("./VEvent");
var VDefinition_1 = require("./VDefinition");
var Components_1 = require("./Components");
var MiddlewareParsers_1 = require("./MiddlewareParsers");
var VTemplate_1 = require("./Templates/VTemplate");
var VWSServer_1 = require("./VWSServer");
var debug = require("debug");
var print = debug("vig:vhandler");
var VHandler = (function () {
    function VHandler(urls, path, prefix) {
        if (urls === void 0) { urls = null; }
        if (path === void 0) { path = ""; }
        if (prefix === void 0) { prefix = ""; }
        this.urls = [];
        this.prefix = "";
        this.parent = null;
        this.children = [];
        this.eventHandler = new VEvent_1.VEvent();
        this.scope = {};
        this.unmuted = {};
        this.urls = urls || [];
        path = path || "";
        this.path = path;
        this.prefix = prefix;
        if (!urls || !urls.length) {
            this.requireFile("urls");
        }
        if (!prefix) {
            this.requireFile("prefix");
        }
        this.config = new Components_1.VConfig(path);
        this.pager = new MiddlewareParsers_1.VPager(path);
        this.event = new Components_1.VEventReader(path);
        this.condition = new MiddlewareParsers_1.VCondition(path);
        this.error = new Components_1.VError(path);
        this.body = new MiddlewareParsers_1.VBody(path);
        this.model = new Components_1.VModel(path);
        this.session = new MiddlewareParsers_1.VSession(path);
        this.middleware = new Components_1.VMiddleware(path);
        this.policy = new MiddlewareParsers_1.VPolicy(path);
        this.router = new Components_1.VRouter(path);
        this.validator = new MiddlewareParsers_1.VValidator(path);
        this.fallback = new MiddlewareParsers_1.VFallback(path);
        this.template = new VTemplate_1.VTemplate(path);
        this.websocket = new Components_1.VWebSocket(path);
        this.definition = new VDefinition_1.VDefinition(path);
        var data = [
            "pager",
            "config",
            "condition",
            "error",
            "body",
            "model",
            "session",
            "event",
            "middleware",
            "policy",
            "event",
            "router",
            "validator",
            "websocket",
            "fallback"
        ];
        for (var _i = 0, data_1 = data; _i < data_1.length; _i++) {
            var key = data_1[_i];
            this[key].loadOn();
        }
        this.updateFallbacks();
        this.loadStaticScope();
        this.initChildren();
    }
    VHandler.prototype.requireFile = function (name) {
        var allowedExt = [".js", ".ts", ".json"];
        for (var _i = 0, allowedExt_1 = allowedExt; _i < allowedExt_1.length; _i++) {
            var ext = allowedExt_1[_i];
            var dir = fsPath.resolve(this.path, "./" + name);
            var resolve = dir + ext;
            if (fs.existsSync(resolve)) {
                var data = require(resolve);
                if (data) {
                    this[name] = data;
                    break;
                }
            }
        }
    };
    VHandler.prototype.setUrls = function (urls) {
        this.urls = urls;
    };
    VHandler.prototype.setParent = function (p) {
        this.parent = p;
        this.template.setParent(p.template);
        this.loadStaticScope();
        for (var _i = 0, _a = this.children; _i < _a.length; _i++) {
            var child = _a[_i];
            child.setParent(this);
            child.loadStaticScope();
        }
    };
    VHandler.prototype.setPrefix = function (prefix) {
        this.prefix = prefix;
    };
    VHandler.prototype.update = function (k, v) {
        if (this[k]) {
            this[k].set(v);
        }
    };
    VHandler.prototype.extend = function (method, cb) {
        return this.router.extend(method, cb);
    };
    VHandler.prototype.set = function (config) {
        var keys = {
            condition: "conditions",
            middleware: "middlewares",
            router: "routers",
            error: "errors",
            config: "configs",
            event: "events",
            body: "bodies",
            model: "models",
            session: "sessions",
            pager: "pagers",
            definition: "definitions",
            policy: "policies",
            websocket: "websockets",
            validator: "validations",
            fallback: "failures"
        };
        if (config.urls) {
            this.urls = config.urls;
        }
        if (config.prefix) {
            this.prefix = config.prefix;
        }
        for (var _i = 0, _a = Object.keys(keys); _i < _a.length; _i++) {
            var key = _a[_i];
            if (config[keys[key]]) {
                this[key].set(config[keys[key]]);
            }
            else {
                this[key].set({});
            }
        }
        this.updateFallbacks();
        this.loadStaticScope();
        this.children = [];
    };
    VHandler.prototype.getScope = function () {
        return this.scope;
    };
    VHandler.prototype.updateFallbacks = function () {
        var keys = {
            validation: "validator"
        };
        var fallbacks = this.fallback.get();
        var items = Object.keys(fallbacks);
        for (var _i = 0, items_1 = items; _i < items_1.length; _i++) {
            var key = items_1[_i];
            var keyOne = this[keys[key]] || this[key];
            if (keyOne) {
                keyOne.setFailureHandler(fallbacks[key]);
            }
        }
    };
    VHandler.prototype.attach = function (app) {
        var _this = this;
        var prefix = this.prefix || "";
        var urls = [];
        if (this.urls instanceof Array) {
            urls = this.urls;
        }
        for (var _i = 0, urls_1 = urls; _i < urls_1.length; _i++) {
            var item = urls_1[_i];
            var url = prefix + item;
            url = url.replace(/\/+/g, "/");
            app.all(url, function (req, res) {
                _this.run(req, res);
            });
        }
        for (var _a = 0, _b = this.children; _a < _b.length; _a++) {
            var child = _b[_a];
            child.attach(app);
        }
    };
    VHandler.prototype.loadStaticScope = function () {
        this.config.parse(this.scope);
        this.error.parse(this.scope);
        this.definition.parse(this.scope);
        if (this.parent) {
            this.parent.loadStaticScope();
            var parent = this.parent.getScope();
            this.scope = _.merge(parent, this.scope);
        }
        this.eventPrepare();
        this.websocketPrepare();
        this.processMuted();
    };
    VHandler.prototype.eventPrepare = function () {
        var _this = this;
        var events = this.event.get();
        for (var key in events) {
            if (key && events[key]) {
                this.eventHandler.on(key, (function (iK) {
                    return function () {
                        var args = [];
                        for (var _i = 0; _i < arguments.length; _i++) {
                            args[_i] = arguments[_i];
                        }
                        return __awaiter(_this, void 0, void 0, function () {
                            return __generator(this, function (_a) {
                                switch (_a.label) {
                                    case 0:
                                        args = [this.scope].concat(args);
                                        return [4, this.event.run(iK, args)];
                                    case 1:
                                        _a.sent();
                                        return [2];
                                }
                            });
                        });
                    };
                })(key));
            }
        }
    };
    VHandler.prototype.websocketPrepare = function () {
        var wss = VWSServer_1.VWSServer.getInstance();
        var websocketHandlers = this.websocket.get();
        for (var key in websocketHandlers) {
            if (key && websocketHandlers[key] instanceof Function) {
                wss.addEventListener(key, this);
            }
        }
    };
    VHandler.prototype.getFixedScope = function () {
        var scope = {
            time: {
                start: new Date()
            }
        };
        for (var _i = 0, _a = Object.keys(this.scope); _i < _a.length; _i++) {
            var key = _a[_i];
            scope[key] = this.scope[key];
        }
        this.model.parse(scope);
        scope.template = this.template;
        return scope;
    };
    VHandler.prototype.checkMutable = function (module) {
        return Object.keys(this[module].get()).length > 0;
    };
    VHandler.prototype.processMuted = function () {
        var mutables = ["body", "session", "condition", "policy", "middleware", "condition", "validator", "pager"];
        for (var _i = 0, mutables_1 = mutables; _i < mutables_1.length; _i++) {
            var key = mutables_1[_i];
            this.unmuted[key] = this.checkMutable(key);
        }
    };
    VHandler.prototype.run = function (req, res) {
        return __awaiter(this, void 0, void 0, function () {
            var _this = this;
            var scope, _a, _b, _c, _d, result, e_1;
            return __generator(this, function (_e) {
                switch (_e.label) {
                    case 0:
                        scope = this.getFixedScope();
                        _e.label = 1;
                    case 1:
                        _e.trys.push([1, 17, , 18]);
                        if (!(req.method === "POST" || this.unmuted.body)) return [3, 3];
                        return [4, this.body.parse(req, res)];
                    case 2:
                        _e.sent();
                        _e.label = 3;
                    case 3:
                        if (!this.unmuted.session) return [3, 5];
                        return [4, this.session.parse(req, res)];
                    case 4:
                        _e.sent();
                        _e.label = 5;
                    case 5:
                        res.vRender = function (data, template, ext) {
                            if (ext === void 0) { ext = "html"; }
                            if (req.session && req.session.user) {
                                data.currentUser = req.session.user;
                            }
                            data.config = scope.configs;
                            res.send(_this.template.render(data, template, ext));
                        };
                        res.errorize = res.restify = function errorize(error, data) {
                            if (!error) {
                                return res.json(scope.errors.UnknownError.restify());
                            }
                            if (error.restify && error.restify instanceof Function) {
                                error = error.restify();
                            }
                            if (!data) {
                                return res.json(_.extend(error));
                            }
                            return res.json(_.extend(error, { data: data }));
                        };
                        _a = this.unmuted.middleware;
                        if (!_a) return [3, 7];
                        return [4, this.middleware.process(req, res, scope)];
                    case 6:
                        _a = (_e.sent()) === false;
                        _e.label = 7;
                    case 7:
                        if (_a) {
                            return [2];
                        }
                        _b = this.unmuted.condition;
                        if (!_b) return [3, 9];
                        return [4, this.condition.process(req, res, scope)];
                    case 8:
                        _b = !(_e.sent());
                        _e.label = 9;
                    case 9:
                        if (_b) {
                            return [2];
                        }
                        _c = this.unmuted.validator;
                        if (!_c) return [3, 11];
                        return [4, this.validator.process(req, res, scope)];
                    case 10:
                        _c = !(_e.sent());
                        _e.label = 11;
                    case 11:
                        if (_c) {
                            return [2];
                        }
                        if (!this.unmuted.pager) return [3, 13];
                        return [4, this.pager.parse(req, res, scope)];
                    case 12:
                        _e.sent();
                        _e.label = 13;
                    case 13:
                        _d = this.unmuted.policy;
                        if (!_d) return [3, 15];
                        return [4, this.policy.process(req, res, scope)];
                    case 14:
                        _d = !(_e.sent());
                        _e.label = 15;
                    case 15:
                        if (_d) {
                            return [2];
                        }
                        return [4, this.router.run(req, res, scope)];
                    case 16:
                        result = _e.sent();
                        if (false === result) {
                            return [2, this.notFound("Not Found!", req, res)];
                        }
                        return [3, 18];
                    case 17:
                        e_1 = _e.sent();
                        print(e_1);
                        return [3, 18];
                    case 18: return [2];
                }
            });
        });
    };
    VHandler.prototype.notFound = function (error, req, res) {
        res.status(404).send("Not Found!");
    };
    VHandler.prototype.wsEvent = function (event, message, ws, req) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4, this.websocket.run(event, message, ws, req, this.scope)];
                    case 1:
                        _a.sent();
                        return [2];
                }
            });
        });
    };
    VHandler.prototype.toJSON = function () {
        var json = {};
        json.prefix = this.prefix;
        json.urls = this.urls;
        json.routers = this.router.toMethods();
        json.middlewares = this.middleware.get();
        json.policies = this.policy.toMethods();
        json.validations = this.validator.get();
        json.conditions = this.condition.get();
        json.failures = this.fallback.get();
        return json;
    };
    VHandler.prototype.initChildren = function () {
        var subDir = "handlers";
        var resolve = fsPath.resolve(this.path, subDir);
        if (!fs.existsSync(resolve)) {
            return;
        }
        var stat = fs.statSync(resolve);
        if (!stat || !stat.isDirectory()) {
            return false;
        }
        this.loadDirectory(resolve);
    };
    VHandler.prototype.loadDirectory = function (dir) {
        var _this = this;
        var files = fs.readdirSync(dir);
        files.forEach(function (file) {
            var absPath = fsPath.resolve(dir, file);
            var stat = fs.statSync(absPath);
            var handler;
            if (stat && stat.isDirectory()) {
                handler = new VHandler(null, absPath);
            }
            else {
                var data = require(absPath);
                handler = new VHandler();
                handler.set(data);
            }
            _this.children.push(handler);
            handler.setParent(_this);
        });
    };
    return VHandler;
}());
exports.VHandler = VHandler;
//# sourceMappingURL=VHandler.js.map